
import re, sys, subprocess, toml
import jinja2, pathlib, pandas, numpy, re
from packaging import version
import datetime
from snakemake import shell

class Report:
    '''
    A class to generate the tables and figures for use in the report.html
    '''
    
    def open_toml(self,tml):

        data = toml.load(tml)

        return data
        
    def write_toml(self, data, output):
        
        with open(output, 'wt') as f:
            toml.dump(data, f)

    def main(self, inputs, workdir, resources):
        '''
        main function of the report class ties it all together
        input:
            :workdir: job directory
            :resources: the directory where templates are stored
            :job_id: the id of the job (for html header)
            :assembler: assembler used - for versions
            :gubbins: not in use yet
            :pipeline: the type of pipeline - default is sa = snippy and assembly
        '''
        
        
        # set up paths variables
        p = pathlib.Path('.')
        reporthtml = pathlib.Path('report.html')
        # path to html template
        indexhtml = pathlib.Path(resources,'index.html') # replace with template
        tml =  self.open_toml(inputs)
        report_template = jinja2.Template(pathlib.Path(indexhtml).read_text())
        reporthtml.write_text(report_template.render(tml))
    

# {input} {params.work_dir} {params.template_path}
inputs = snakemake.input
resources = snakemake.params.template_path
workdir = snakemake.params.work_dir
report = Report()
report.main(inputs = inputs, resources=resources, workdir=workdir)


import toml, pathlib, subprocess, sys

from snakemake import shell

def generate_asm_cmd(assembler, r1, r2, isolate, threads = 4, memory = 8):
    
    if assembler == 'shovill':
        cmd = f"shovill --R1 {r1} --R2 {r2} --outdir {isolate}/shovill --cpus {threads} --ram {memory} --force"
    elif assembler == 'spades':
        cmd = f"spades.py -1 {r1} -2 {r2} -o {isolate}/spades --threads {threads} --memory {memory}"
    else:
        cmd = f"skesa --fastq {r1},{r2} --contigs_out {isolate}/contigs.fa --cores {threads} --memory {memory}"
    print(cmd)
    return cmd


def generate_cmd(prefill, r1, r2, isolate, assembler, data):
    
    prfl = pathlib.Path(prefill, isolate, 'contigs.fa')
    if prfl.exists():
        cmd = f"cp {prfl} {isolate}/contigs.fa"
        data[isolate]['assembly']['source'] = f"Prefilled from: {prfl}"
    else:
        cmd = generate_asm_cmd(assembler = assembler, r1 = r1, r2 = r2, isolate = isolate)
        data[isolate]['assembly']['source'] = cmd
    
    return cmd, data

def generate_mv_cmd(assembler, isolate):

    cmd = f"mv {isolate}/{assembler}/contigs.fa {isolate}/contigs.fa"
    return cmd

def run_cmd(cmd):
    
    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')
    return p.returncode
   

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(seqdata, isolate, assembler, prefill):
    
    # set up data dict
    seqdata = open_toml(tml = seqdata)
    r1 = seqdata[isolate]['seqdata']['R1']
    r2 = seqdata[isolate]['seqdata']['R2']
    data = {}
    data[isolate] = {}
    data[isolate]['assembly'] = {}
    # run kraken
    # data[isolate]['seqdata']['data']['Quality']
    if seqdata[isolate]['seqdata']['data']['Quality'] == 'PASS':
        cmd, data = generate_cmd(prefill = prefill, r1 = r1, r2 = r2, isolate = isolate, assembler =assembler, data = data)
        assembly_returncode = run_cmd(cmd)
        if assembly_returncode == 0:
            p = run_cmd(generate_mv_cmd(assembler = assembler, isolate = isolate))
            # add to data dict
            data[isolate]['assembly']['done'] = 'Yes'
            data[isolate]['assembly']['assembler'] = assembler
    else:
        data[isolate]['assembly']['done'] = 'No'
        data[isolate]['assembly']['assembler'] = f"Assembly not performed - failed QC"

    write_toml(data = data, output= f'{isolate}/assembly.toml')

seqdata = snakemake.input
isolate = snakemake.wildcards.sample
assembler = snakemake.params.assembler
prefill= snakemake.params.prefill_path

main(seqdata = seqdata, isolate = isolate,  assembler = assembler, prefill = prefill)


import toml, pathlib, subprocess, sys
from snakemake import shell

def generate_snippy_cmd(r1, r2, isolate, reference, threads):
    
    p = pathlib.Path(isolate)
    cmd = f"snippy --outdir {isolate} --ref {reference} --R1 {r1} --R2 {r1} --force --cpus {threads}"
    
    return cmd

def run_cmd(cmd):
    
    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')

    return p.returncode

def get_reads(inputs, isolate):

    s = open_toml(inputs)
    r1 = s[isolate]['seqdata']['R1']
    r2 = s[isolate]['seqdata']['R2']
    return r1,r2

def get_quality(inputs, isolate):

    s = open_toml(inputs)
    if s[isolate]['seqdata']['data']['Quality'] == 'PASS':
        return 'Yes'
    else:
        return 'No'


def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs, isolate, output, reference, threads):
    
    r1,r2 = get_reads(inputs = inputs, isolate = isolate)
    run_snippy = get_quality(inputs = inputs, isolate = isolate)
    data = {}
    data[isolate] = {}
    data[isolate]['snippy'] = {}
    data[isolate]['snippy']['reference'] = reference
    data[isolate]['snippy']['R1'] = r1
    data[isolate]['snippy']['R2'] = r2
    data[isolate]['snippy']['run_snippy'] = run_snippy

    if run_snippy == 'Yes':
        cmd = generate_snippy_cmd(r1 = r1, r2=r2, isolate = isolate, reference = reference, threads = threads)
        p = run_cmd(cmd)
        if p == 0:
            data[isolate]['snippy']['alignment'] = f"{isolate}/snps.aligned.fa"
            data[isolate]['snippy']['vcf'] = f"{isolate}/snps.vcf"
            data[isolate]['snippy']['cmd'] = cmd
            data[isolate]['snippy']['done'] = 'Yes'
        else:
            data[isolate]['snippy']['done'] = 'No'
    write_toml(data = data, output = f"{isolate}/snippy.toml") 

inputs = snakemake.input
isolate = snakemake.wildcards.sample
output = snakemake.output
reference = snakemake.params.reference
threads = snakemake.threads

main(inputs = inputs, isolate = isolate, output = output,reference = reference, threads =threads)



import toml, pathlib, subprocess, sys
from snakemake import shell

def generate_prokka_cmd(isolate, assembly):
    
    cmd =  f"prokka --outdir {isolate} --prefix {isolate} --mincontiglen 500 --notrna --fast --force {assembly} --cpus 4"

    return cmd

def generate_rm_cmd(isolate):

    cmd = f"rm {isolate}/*.err {isolate}/*.faa {isolate}/*.ffn {isolate}/*.fsa {isolate}/*.sqn {isolate}/*.tbl {isolate}/*.tsv"
    return cmd

def run_cmd(cmd):
    
    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')
    return p.returncode
   

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs, isolate, seqdata):
    
    # set up data dict
    data = open_toml(inputs)
    # data[isolate] = {}
    data[isolate]['prokka'] = {}
    # run kraken
    assembly = f"{isolate}/contigs.fa"
    seqdata = open_toml(seqdata)
    cmd = generate_prokka_cmd(isolate = isolate, assembly = assembly)
    if seqdata[isolate]['seqdata']['data']['Quality'] == 'PASS':
        p = run_cmd(cmd)
        if p == 0:
            rm_cmd = generate_rm_cmd(isolate = isolate)
            r = generate_rm_cmd(run_cmd)
            data[isolate]['prokka']['done'] = True
            data[isolate]['prokka']['gff'] = f'{isolate}/{isolate}.gff'
            data[isolate]['prokka']['txt'] = f'{isolate}/{isolate}.txt'
    else:
        data[isolate]['prokka']['done'] = False

    
    write_toml(data = data, output= f'{isolate}/prokka.toml')

inputs = snakemake.input.assembly
isolate = snakemake.wildcards.sample
seqdata = snakemake.input.seqdata

main(inputs = inputs, isolate = isolate, seqdata = seqdata)
    




from Bio import SeqIO
import pathlib, toml, pandas, sys
from snakemake import shell


def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)


def fa(inputs, isolate, min_size =500):
    p = pathlib.Path(f"{inputs}")
    min_len = 1.0e+10
    max_len = 0
    contig_length = []
    Ns = 0
    gaps = 0
    contigs_less_than_min = 0
    min_size = int(min_size)
    no = 0
    for i in SeqIO.parse(f"{p}", 'fasta'):
            contig_length.append(len(i.seq))
            if len(i.seq) < min_size:
                    contigs_less_than_min += 1
            else:   
                    min_len = len(i.seq) if len(i.seq) < min_len else min_len
                    max_len = len(i.seq) if len(i.seq) > max_len else max_len
                    Ns = Ns + i.seq.lower().count('n')
                    gaps = gaps + i.seq.count('-')
                    no += 1
    length = sum([c for c in contig_length if c > min_size])
    len_sorted = sorted([c for c in contig_length if c > min_size])
    avg_len = int(length/ len([c for c in contig_length if c > min_size]))
    cum = 0 
    N50 = 0
    for c in len_sorted:
            # print(c)
            cum += c
            # print(cum)
            if cum >= length/2:
                    N50 = c
                    break

    data = {'Name': isolate, 'bp':length, '# Contigs':no, 'Ns':Ns, '# Gaps':gaps, 'Min Contig size':min_len, 'Max Contig size':max_len,  'Avg Contig size':avg_len, 'N50':N50, 'Quality': 'PASS'}
    return(data)

def main(inputs,isolate):
#     colnames = ['Name','bp','# Contigs','Ns','# Gaps','Min Contig size','Max Contig size','Avg Contig size','N50']
    # print(colnames)
    # print('\t'.join(colnames))
    assembly_data = open_toml(tml = inputs)

    if assembly_data[isolate]['assembly']['done'] == 'Yes':
        # print('in true')
        assembly = f"{pathlib.Path(isolate, 'contigs.fa')}"
        d = fa(inputs=assembly,isolate= isolate)
        data = {}
        data[isolate]={}
        data[isolate]['assembly_stats'] = d
        
    else:
        data = {}
        data[isolate]={}
        data[isolate]['assembly_stats'] = {'Name': isolate, 'bp':'-', '# Contigs':'-', 'Ns':'-', '# Gaps':'-', 'Min Contig size':'-', 'Max Contig size':'-',  'Avg Contig size':'-', 'N50':'-', 'Quality': 'ASSEMBLY NOT PERFORMED please see Sequence data tab'}
    write_toml(data= data, output = f'{isolate}/assembly_stats.toml')
        
 
inputs = snakemake.input
isolate = snakemake.wildcards.sample

main(inputs = inputs, isolate = isolate)
    





import toml, pathlib, subprocess, sys, pandas
from snakemake import shell

def generate_triangle_cmd():
    
    cmd = f"mash triangle -C */sketch.msh > preview_distances.tab"
    return cmd

def generate_tree_cmd():

    cmd = f"quicktree -in m -out t preview_distances.tab  | nw_order -c n - > preview.newick"
    return cmd

def run_cmd(cmd):
    
    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')
    return p.returncode

def get_isolates(tab):
    data = open(tab).readlines()
    isolates = ['mash-dist']
    for d in range(1,len(data)):
        l = data[d].split()
        isolates.append(l[0])
    return isolates

def clean_dist_mat(tab):
    isolates = get_isolates(tab)
    df = pandas.read_csv(tab, names = isolates, sep = '\t', header = None, skiprows = [0])
    df = df.fillna('-')
    df.to_csv(tab, sep = '\t', index = False)

def open_toml(tml):
    # print(tml)
    data = toml.load(tml)

    return data
    
def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs):
    
    triangle = generate_triangle_cmd()
    # print(triangle)
    p = run_cmd(triangle)
    # print(p)
    if p == 0:
        tree = generate_tree_cmd()
        # print(tree)
        q = run_cmd(tree)
        if q == 0:
            data = {}
            data['preview'] = {}
            data['preview']['tree'] = 'preview.newick'
            data['preview']['distances'] = 'preview_distances.tab'
            clean_dist_mat(data['preview']['distances'])
            write_toml(data = data, output = 'preview.toml')
    
    
inputs = snakemake.input

main(inputs = inputs)

# if __name__ == '__main__':
    
#     main(inputs = sys.argv[1:])
    



# mash triangle -C *.msh

# mash sketch -m 5 -s 10000 -r -o 2019-12803-6/sketch -I 2019-12803-6 -C 2019-12803-6/R1.fq.gz 2019-12803-6/R1.fq.gz


import toml, pathlib, subprocess, sys, pandas, datetime
from snakemake import shell

def generate_snippy_clean_cmd():

    cmd = f"snippy-clean_full_aln core.full.aln  > clean.full.aln"								
    return cmd

def generate_gubbins_cmd():

    cmd = f"run_gubbins.py -c 36  --prefix core clean.full.aln"								
    return cmd

def generate_snp_sites_cmd():
    
    cmd = f"run_gubbins.py -c 36  --prefix core gubbins.aln"								
    return cmd

def run_cmd(cmd):

    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')
    return p.returncode

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs, gubbins):
    data = {}
    data['gubbins'] = {}
    # print(gubbins)
    if gubbins != False:
        clean = run_cmd(generate_snippy_clean_cmd())
        if clean == 0:
            gub = run_cmd(generate_gubbins_cmd())
            if gub == 0:
                sites = run_cmd(generate_snp_sites_cmd())
                if sites == 0:
                    data['gubbins']['run_gubbins'] = 'Yes'
                    data['gubbins']['aln'] = 'gubbins.aln'
    else:
        data['gubbins']['run_gubbins'] = 'No'

        
    write_toml(data = data, output = "gubbins.toml")

inputs = snakemake.input
gubbins = snakemake.params.gubbins
main(inputs = inputs, gubbins = gubbins)


# mash triangle -C *.msh

# mash sketch -m 5 -s 10000 -r -o 2019-12803-6/sketch -I 2019-12803-6 -C 2019-12803-6/R1.fq.gz 2019-12803-6/R1.fq.gz


import toml, pathlib, subprocess, sys, pandas
from snakemake import shell

def combine_resistome(inputs):

    tab = pandas.DataFrame()

    for i in inputs:
        tml = open_toml(i)
        isolate = list(tml.keys())[0]

        if tml[isolate]['resistome']['done']:
            df = pandas.DataFrame(tml[isolate]['resistome']['data'], index = [0])
            df['Quality'] = 'PASS'
        else:
            df = pandas.DataFrame({'Isolate':isolate, 'Quality': 'NOT INCLUDED - FAIL QC'}, index = [0])
            # df['Quality '] = 'NOT INCLUDED - FAIL QC'
        if tab.empty:
            tab = df
        else:
            tab = tab.append(df, sort = True)
    tab = tab.set_index('Isolate')
    test = tab[tab['Quality'] == 'PASS']
    if not test.empty:
        tab.to_csv('resistome.tab', sep= '\t', index = True)
        return tab
    else:
        return tab

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs):
    
    # set up data dict
    
    data = {}
    tab = combine_resistome(inputs)
    if tab.empty:
        data['resistome'] = 'resistome not performed'
    else:
        data['resistome'] = tab.to_dict(orient= 'records')
    write_toml(data = data, output= f'resistome.toml')

inputs = snakemake.input

main(inputs = inputs)
    




import toml, pathlib, subprocess, sys, pandas, json
from snakemake import shell

def get_gffs(inputs):

    l = []
    for i in inputs:
        tml = open_toml(i)
        isolate = list(tml.keys())[0]
        if tml[isolate]['prokka']['done']:
            gff = tml[isolate]['prokka']['gff']
            l.append(gff)
    return ' '.join(l)

def generate_roary_cmd(gffs):

    cmd = f"nice roary -p 36 -f roary {gffs}"
    return cmd

def generate_mv_cmd():
    
    cmd = f"mv roary_*/* roary"
    return cmd

def generate_rm_cmd():
    
    cmd = f"rm -r roary_*"
    return cmd

def run_cmd(cmd):
    
    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')
    return p.returncode

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs):
    
    gffs = get_gffs(inputs)
    data = {}
    data['roary'] = {}
    
    roary = run_cmd(generate_roary_cmd(gffs))
    if roary == 0:
        run_cmd(generate_mv_cmd())
        run_cmd(generate_rm_cmd())

        data['roary']['done'] = True
        data['roary']['csv'] = "roary/gene_presence_absence.csv"
        data['roary']['txt']="roary/summary_statistics.txt"
    else:
        data['roary']['done'] = False
    
    write_toml(data = data, output= f'roary.toml')

inputs = snakemake.input.prokka

main(inputs = inputs)
    




import toml, pathlib, subprocess, sys, pandas, numpy

from snakemake import shell
import pandas, pathlib


def assembly_combine(assemblies):

    ac = pandas.DataFrame()
    
# colnames = ['Name','bp','# Contigs','Ns','# Gaps','Min Contig size','Max Contig size','Avg Contig size','N50']
# {'Name': isolate, 'bp':length, '# Contigs':no, 'Ns':Ns, '# Gaps':gaps, 'Min Contig size':min_len, 'Max Contig size':max_len,  'Avg Contig size':avg_len, 'N50':N50}
    for a in assemblies:
        d = open_toml(a)
        isolate=list(d.keys())[0]
        df = pandas.DataFrame(d[isolate]['assembly_stats'], index = [0])
        if ac.empty:
            ac = df
        else:
            ac = df.append(ac, sort = True)
    # print(ac)
    return ac


def combine(prokka, assembly_stats):
# 
    
    gff = pandas.DataFrame()
    
    for p in prokka:
        tml = open_toml(p)
        isolate=list(tml.keys())[0]
        if tml[isolate]['prokka']['done'] and 'txt' in tml[isolate]['prokka']:
            g = pathlib.Path(tml[isolate]['prokka']['txt'])
            df = pandas.read_csv(g, sep = ':', header = None, names = ['cond', f"{g.parts[0]}"])
            # print(df)
            if gff.empty:
                    gff = df
            else:
                    gff = gff.merge(df, how = 'outer')
    gff = gff[gff['cond'].isin(['CDS', 'rRNA'])]
    gff = gff.T
    gff.columns = gff.iloc[0]
    gff = gff.iloc[1:]  
    assembly = assembly_stats.merge(gff, left_on = ['Name'], right_on= gff.index, how = 'outer')

    m = list(assembly[assembly['# Contigs'] != '-']['# Contigs'])  
    mn = numpy.mean(m)
    s =(2* numpy.std(m))
    cut = mn + s
    assembly['Quality'] = numpy.where(assembly['# Contigs'].replace('-',0) <= cut, assembly['Quality'], f'WARNING - assembly outlier (> {round(cut,2)} contigs)')
    assembly = assembly.rename(columns={'Name':'Isolate'})
    
    assembly = assembly[['Isolate', 'bp','# Contigs','Ns','# Gaps','Min Contig size','Max Contig size','Avg Contig size','N50', 'CDS','rRNA', 'Quality' ]]

    assembly.to_csv(f"assembly.tab", sep = '\t', index = False)

    return assembly.to_dict(orient = 'records')

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(prokka):
    
    assemblies = assembly_combine(assemblies = prokka)
    assembly_stat_dict = combine(prokka = prokka, assembly_stats = assemblies) 
    # set up data dict
    data = {}
    data['assembly_stat'] = assembly_stat_dict
    write_toml(data = data, output= f'assembly.toml')


prokka = snakemake.input.prokka

main(prokka = prokka)
    




import toml, pathlib, subprocess, sys
from snakemake import shell

def generate_mv_cmd(iostring):

    cmd = f"mv {iostring} && mv *.tab report/"
    return cmd

def get_io(inputs):

    i = f"{inputs}"
    o = f"{pathlib.Path('report', inputs)}" if f"{inputs}" != 'report.html' else f"{pathlib.Path('report', 'index.html')}"
    return f"{i} {o}"

def run_cmd(cmd):
    
    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')
    return p.returncode

    
def main(inputs):
    
    for i in inputs:
        iostring = get_io(i)
        cmd = generate_mv_cmd(iostring)
        p = run_cmd(cmd)
    


inputs = snakemake.input    
main(inputs = inputs)
    




import toml, pathlib, subprocess, sys, pandas, json
from snakemake import shell

def generate_mlst_cmd(assembly, isolate):

    cmd = f"mlst --label {isolate} --json {isolate}/{isolate}.json {assembly}"
    return cmd

def extract_mlst(isolate):

    jsn = json.load(open(f"{isolate}/{isolate}.json"))
    isolate = jsn[0]['id']
    alleles = []
    ax = zip(jsn[0]['alleles'].keys(), jsn[0]['alleles'].values())
    for a in ax:
        alleles.append(f"{a[0]}({a[1]})")
    alleles = sorted(alleles)

    d = {
        'Isolate':isolate,
        'alleles': alleles,
        'len_alleles': len(alleles),
        'scheme':jsn[0]['scheme'],
        'ST':jsn[0]['sequence_type']
    }
    return d

def generate_rm_cmd(isolate):

    cmd = f"rm {isolate}/{isolate.json}"
    return df.to_dict(orient = 'records')

def run_cmd(cmd):
    
    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')
    return p.returncode

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs, isolate, seqdata):
    
    data = {}
    data[isolate] = {}
    data[isolate]['mlst'] = {}
    seqdata = open_toml(seqdata)
    assembly = f"{pathlib.Path(isolate, 'contigs.fa')}"
    if seqdata[isolate]['seqdata']['data']['Quality'] == 'PASS':
    # set up data dict
        cmd = generate_mlst_cmd(assembly = assembly, isolate = isolate)
        p = run_cmd(cmd)
        if p == 0:
            data[isolate]['mlst']['done'] = 'Yes'
            data[isolate]['mlst']['data'] = extract_mlst(isolate)
    else:
            data[isolate]['mlst']['done'] = 'No'
            data[isolate]['mlst']['data'] = f"MLST not performed - failed QC"
            
    write_toml(data = data, output= f'{isolate}/mlst.toml')

inputs =snakemake.input.assembly
isolate = snakemake.wildcards.sample
seqdata = snakemake.input.seqdata

main(inputs = inputs, isolate = isolate, seqdata = seqdata)
    




import toml, pathlib, subprocess, sys, pandas
from snakemake import shell

def combine_mlst(inputs):

    tab = pandas.DataFrame()

    for i in inputs:
        tml = open_toml(i)
        isolate = list(tml.keys())[0]
        d = {
            "Isolate": isolate,
            "Scheme" : '',
            "ST": '',
            "Quality": 'PASS'
        }
        if tml[isolate]['mlst']['done'] == 'Yes' and tml[isolate]['mlst']['data']['scheme'] != '-':
            d['Scheme'] = tml[isolate]['mlst']['data']['scheme']
            d['ST'] = int(tml[isolate]['mlst']['data']['ST']) if tml[isolate]['mlst']['data']['ST'] != '-' else '-'
            for a in range(len(tml[isolate]['mlst']['data']['alleles'])):
                d[f"Allele {a+1}"] = tml[isolate]['mlst']['data']['alleles'][a]
            
            
        elif tml[isolate]['mlst']['done'] == 'Yes' and tml[isolate]['mlst']['data']['scheme'] == '-':
            d["Scheme"] = "No scheme available"
        elif tml[isolate]['mlst']['done'] == 'No':
            d['Quality'] = 'NOT INCLUDED - FAIL QC'
        df = pandas.DataFrame(d, index = [0])
        if tab.empty:
            tab = df
        else:
            tab = tab.append(df, sort = True)
    tab = tab.set_index('Isolate')
    cols = ['Scheme', 'ST']
    cols.extend([a for a in tab.columns if 'Allele' in a])
    cols.append('Quality')
    tab = tab[cols]
    tab.to_csv('mlst.tab', sep= '\t', index = True)
    return tab

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs):
    
    # set up data dict
    tab = combine_mlst(inputs)
    data = {}
    data['mlst'] = tab.to_dict(orient= 'records')
    write_toml(data = data, output= f'mlst.toml')

inputs = snakemake.input
main(inputs = inputs)
    




"""Bohra 
.. moduleauthor:: Kristy Horan <kristyhoran15@gmail.com>

Bohra exinct tree kangaroo that lived on the nullarbor

Bohra is microbial genomics pipeline, designed predominantly for use in public health, but may also be useful in research settings. The pipeline takes as input a tab-delimited file with the isolate IDs followed by the path to READ1 and READ2, a reference for alignment and a unique identifier, where reads are illumina paired end reads (other platforms are not supported).

The pipline is based on nullarbor (https://github.com/tseemann/nullarbor) and is designed to be run in high performance computing environment.

Bohra is modular allowing the user to choose between calling SNPs and generating a phylogenetic tree, perform assemblies and detect AMR, perform typing etc; Or use the full pipeline to call SNPs, generate phylogenies, assemble, type and detect the pan-geneome. The output of Bohra is a html report that can be distributed, with downloadable tables and data.
"""


import logging
import argparse
import configargparse
import pathlib
import sys
import os
import shutil
from bohra.SnpDetection import RunSnpDetection
# from bohra.ReRunSnpDetection import ReRunSnpDetection
from bohra.Utils import Nulla2bohra, UpdateBohra, CheckDeps
from bohra.version import version


#logging.basicConfig(filename='job.log',level=logging.INFO, format='[%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')

def run_pipeline(args):
    '''
    Run the pipeline for the first time
    '''
    R = RunSnpDetection(args)
    return(R.run_pipeline())

def rerun_pipeline(args):
    '''
    Rerun the pipeline on a previous dataset, adding, removing isolates or changing reference or mask file
    '''
    R = ReRunSnpDetection(args)
    return(R.run_pipeline())

def nulla2bohra(args):
    '''
    ensure that bohra can be rerun over the top of an existing nullarbor directory
    '''
    N = Nulla2bohra(args)
    return(N.update())

def check_deps(args):
    """
    check that dependencies are all installed
    """
    C = CheckDeps(args)
    return(C.check())
    
def main():
    # setup the parser
  
    parser = configargparse.ArgumentParser(description=f'Bohra - a bacterial genomics pipeline - version {version}',formatter_class=configargparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + version)
    # subparser for running the pipeline
    subparsers = parser.add_subparsers(help="Task to perform")

    

    parser_sub_run = subparsers.add_parser('run', help='Initial run of Bohra', formatter_class=configargparse.ArgumentDefaultsHelpFormatter,default_config_files=[f"{pathlib.Path.cwd().absolute() / 'bohra.conf'}"])
    
    # options for running
    parser_sub_run.add_argument('--input_file','-i',help='Input file = tab-delimited with 3 columns <isolatename>  <path_to_read1> <path_to_read2>', default='')
    parser_sub_run.add_argument('-S', '--use_singularity', action='store_true', help = 'Set if you would like to use singularity containers to run bohra.')
    # parser_sub_run.add_argument('--snippy_singularity', default='docker://mduphl/snippy:v4.6.0', help='The path to containers. If you want to use locally stored contianers please pull from dockerhub://mduphl/<toolname>.')
    parser_sub_run.add_argument('--abritamr_singularity', default='docker://mduphl/abritamr:v0.2.2', help='The path to containers. If you want to use locally stored contianers please pull from dockerhub://mduphl/<toolname>.')
    parser_sub_run.add_argument('--job_id','-j',help='Job ID, will be the name of the output directory', default='')
    parser_sub_run.add_argument('--reference','-r',help='Path to reference (.gbk or .fa)', default = '')
    parser_sub_run.add_argument('--mask','-m',default = False, help='Path to mask file if used (.bed)')
    parser_sub_run.add_argument('--kraken_db', '-k', env_var="KRAKEN2_DEFAULT_DB", help="Path to DB for use with kraken2, if no DB present speciation will not be performed.")
    parser_sub_run.add_argument('--pipeline','-p', default = 'preview', choices=['preview','sa','all'], help=f"The pipeline to run. Preview (--preview - default) will calculate mash-distances and a mash-tree for quick inspection of your dataset. SNPs and ASSEMBLIES ('sa') will perform SNPs and ASSEMBLIES. ALL ('all') will perform SNPS, ASSEMBLIES and ROARY for pan-genome analysis")
    parser_sub_run.add_argument('--assembler','-a', default = 'shovill', choices=['shovill','skesa','spades'], help=f"Assembler to use.")
    parser_sub_run.add_argument('--cpus','-c',help='Number of CPU cores to run, will define how many rules are run at a time', default=16)
    parser_sub_run.add_argument('--minaln','-ma',help='Minimum percent alignment. Isolates which do not align to reference at this threshold will not be included in core phylogeny.', default=80)
    parser_sub_run.add_argument('--mincov','-mc',help='Minimum percent alignment. Isolates which do not have average read coverage above this threshold will not be included further analysis.', default=40)
    parser_sub_run.add_argument('--prefillpath','-pf',help='Path to existing assemblies - in the form path_to_somewhere/isolatename/contigs.fa')
    parser_sub_run.add_argument('-mdu', action = "store_true", help='If running on MDU data')
    parser_sub_run.add_argument('-workdir','-w', default = f"{pathlib.Path.cwd().absolute()}", help='The directory where Bohra will be run, default is current directory')
    parser_sub_run.add_argument('-resources','-s', default = f"{pathlib.Path(__file__).parent / 'templates'}", help='Directory where templates are stored')
    parser_sub_run.add_argument('-force','-f', action="store_true", help = "Add if you would like to force a complete restart of the pipeline. All previous logs will be lost.")
    parser_sub_run.add_argument('-dry-run','-n', action="store_true", help = "If you would like to see a dry run of commands to be executed.")
    parser_sub_run.add_argument('--cluster', action="store_true", help = "If you are running Bohra on a cluster.")
    parser_sub_run.add_argument('--json',help='Path to cluster.json - required if --cluster is set', default='')
    parser_sub_run.add_argument('--queue',help='Type of queue (sbatch or qsub currently supported) - required if --cluster is set.', default='')
    parser_sub_run.add_argument('--gubbins', '-g', action = 'store_true', help = 'Set to use gubbins for recombination correction.')
    parser_sub_run.add_argument('--keep', default = 'N', choices= ['Y', 'N'], help = 'If you are rerunning bohra over an exisiting directory set --keep to \'Y\' to archive report files - otherwise previous reprot files will be removed.')
    
    # parser for update
    parser_sub_nulla2bohra = subparsers.add_parser('nulla2bohra', help='Ensure that bohra can be rerun over an existing nullarbor folder. Can also be used to update older bohra directories. Must supply name of nullarbor directory, and your isolates.tab file', formatter_class=configargparse.ArgumentDefaultsHelpFormatter,default_config_files=[f"{pathlib.Path.cwd().absolute() / 'bohra.conf'}"])

    parser_sub_nulla2bohra.add_argument('-workdir','-w', default = f"{pathlib.Path.cwd().absolute()}", help='Working directory, default is current directory')
    parser_sub_nulla2bohra.add_argument('--job_id','-j',help='Job directory - the --name you used for nullarbor, will be the name of the output directory', default='')
    parser_sub_nulla2bohra.add_argument('--input_file','-i',help='Input file = tab-delimited with 3 columns <isolatename>  <path_to_read1> <path_to_read2>', default='')
    parser_sub_nulla2bohra.add_argument('--reference', '-r', help = 'The reference that was used in the previous run/ nullarbor job', default = '')
    
    # parser for checks
    parser_sub_check = subparsers.add_parser('check', help = "Check that all dependencies for bohra have been installed.")
    parser_sub_check.add_argument('--kraken_db', '-k', env_var="KRAKEN2_DEFAULT_DB", help="Path to DB for use with kraken2, if no DB present speciation will not be performed.")
    parser_sub_run.set_defaults(func=run_pipeline)
    parser_sub_nulla2bohra.set_defaults(func=nulla2bohra)
    parser_sub_check.set_defaults(func = check_deps)


        
    args = parser.parse_args()
    
    if len(sys.argv) == 2 and sys.argv[1] in ['run', 'nulla2bohra']:
        parser.print_help(sys.stderr)
    else:
        logger = logging.getLogger(__name__)
        logger.setLevel(logging.INFO)
        # create file handler which logs even debug messages
        fh = logging.FileHandler('bohra.log')
        fh.setLevel(logging.INFO)
        # create console handler with a higher log level
        ch = logging.StreamHandler()
        ch.setLevel(logging.INFO)
        # create formatter and add it to the handlers
        formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)
        # add the handlers to the logger
        logger.addHandler(ch)
        logger.addHandler(fh)
        logger.info(f"Starting bohra pipeline using {' '.join(sys.argv)}")
        args.func(args)
	
if __name__ == '__main__':
    main()




import toml, pathlib, subprocess, sys, pandas
from snakemake import shell

def combine_tomls(inputs, isolate):

    final_toml = {}
    final_toml[isolate] = {}
    for i in inputs:
        tml = open_toml(i)
        # print(st)
        t = list(tml[isolate].keys())[0]
        final_toml[isolate][t] = tml[isolate][t]
        # subprocess.run(f"rm {i}", shell = True, capture_output = True, encoding = "utf-8")
    
    return final_toml

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs, isolate):
    
    # set up data dict
    final_toml = combine_tomls(inputs = inputs, isolate = isolate)
    
    write_toml(data = final_toml, output= f'{isolate}/final.toml')

isolate = snakemake.wildcards.sample
inputs = snakemake.input

main(isolate = isolate, inputs = inputs)
    




import toml, pathlib, subprocess, sys, pandas, json 
from snakemake import shell

def get_roary(inputs):

    tml = open_toml(inputs)
    return tml['roary']['csv']

def generate_svg_cmd(csv, output):

    cmd = f"perl roary2svg.pl {csv} > {output}"
    return cmd

def run_cmd(cmd):
    
    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')
    return p.returncode

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs):
    
    csv = get_roary(inputs)
    data = {}
    data['pan_figure'] = {}
    data['pan_figure']['figure'] = "pan_genome.svg"   
    fig = run_cmd(generate_svg_cmd(csv = csv, output = data['pan_figure']['figure']))
    if fig == 0:
        data['pan_figure']['done'] = True
    else:
        data['pan_figure']['done'] = False
    write_toml(data = data, output= f'pan_genome.toml')


inputs = snakemake.input
main(inputs = inputs)
    




import toml, pathlib, subprocess, sys, pandas
from snakemake import shell

def generate_abritamr_cmd(input_file, isolate, wd, job_id):
    
    w = pathlib.Path(wd, isolate)
    cmd = f"abriTAMR -c {input_file} -pfx {isolate} -w {w}"
    return cmd

def generate_dict(isolate):

    df = pandas.read_csv(f"{isolate}/summary_matches.csv")
    return df.to_dict(orient = 'records')

def run_cmd(cmd):
    
    p = subprocess.run(cmd, shell = True, capture_output=True, encoding = 'utf-8')
    return p.stderr

def open_toml(tml):

    data = toml.load(tml)

    return data

def write_toml(data, output):
    
    with open(output, 'wt') as f:
        toml.dump(data, f)
    
def main(inputs, isolate, seqdata, wd, job_id):
    
    seqdata = open_toml(seqdata)
    data = {}
    data[isolate] = {}
    data[isolate]['resistome'] = {}
    if seqdata[isolate]['seqdata']['data']['Quality'] == 'PASS':
    # set up data dict
        tml = open_toml(inputs)
        contigs = f"{isolate}/contigs.fa"
        cmd = generate_abritamr_cmd(input_file = contigs, isolate = isolate, wd = wd, job_id = job_id)
        p = run_cmd(cmd)
        data[isolate]['resistome']['tool'] = 'abritamr'
        
        if f"pipeline successfully completed" in p:
            data[isolate]['resistome']['data'] = generate_dict(isolate)
            data[isolate]['resistome']['done'] = True
        else:
            data[isolate]['resistome']['done'] = False
    else:
        data[isolate]['resistome']['done'] = False
        data[isolate]['resistome']['tool'] = f"AMR not performed - failed QC"
    
    write_toml(data = data, output= f'{isolate}/resistome.toml')

inputs = snakemake.input.assembly
isolate = snakemake.wildcards.sample
seqdata = snakemake.input.seqdata
wd = snakemake.params.work_dir
job_id = snakemake.params.job_id

main(inputs = inputs, isolate = isolate, seqdata = seqdata, wd = wd, job_id = job_id)